﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Diagnostics;
using System.Collections;

namespace TrakalMatej.Astar {
    public class Astar<T> : IEnumerable {
        Point[] tahy;
        Bod start = null;
        Bod cil = null;
        int sirkaPole = 0, vyskaPole = 0;
        bool nalezenaCesta = false;
        bool jizSeHledalo = false;
        LinkedList<Bod> cesta = null;
        public int Heuristika { get; set; }
        Matice mapa;
        List<Bod> otevrene = new List<Bod>();
        Bod otevrenyBod;

        public Astar(Matice mapa, Point[] tahy, int heuristika) {
            this.mapa = mapa;
            this.tahy = tahy;
            this.sirkaPole = mapa.DejMatici().GetLength(0);
            this.vyskaPole = mapa.DejMatici().GetLength(1);
            this.start = mapa.Start;
            this.cil = mapa.Cil;
            this.Heuristika = heuristika;
            VymazCesty();
        }
        public void VymazCesty() {
            for (int y = 0; y < vyskaPole; y++) {
                for (int x = 0; x < sirkaPole; x++) {
                    mapa[x, y].Rodic = null;
                    mapa[x, y].Uzavren = false;
                    mapa[x, y].OhodnoceniGVzdalenostStart = 0;
                    mapa[x, y].OhodnoceniHNejCestaDoCile = 0;
                    mapa[x, y].OhodnoceniFNejCestaStartCil = 0;
                    mapa[x, y].PoziceNaCeste = 0;
                    if (mapa[x, y].Data != HodnotaBodu.cil && mapa[x, y].Data != HodnotaBodu.start && mapa[x, y].Data != HodnotaBodu.plne) {
                        mapa[x, y].Data = HodnotaBodu.prazdne;
                    }
                }
            }
        }
        private void vypisDebug(Bod bod, params string[] popis) {
            Debug.WriteLine(String.Format(((popis.Length == 1) ? popis[0] + " - " : "") + "{6,-3} POS: {0,4}:{1,-4} G:{2,8:##0.000} H:{3,8:##0.000} F:{4,8:##0.000} {5,-8} ",
                bod.Souradnice.X.ToString(),
                bod.Souradnice.Y.ToString(),
                bod.OhodnoceniGVzdalenostStart,
                bod.OhodnoceniHNejCestaDoCile,
                bod.OhodnoceniFNejCestaStartCil,
                bod.Data,
                bod.PoziceNaCeste)
                );
        }
        private bool jePristupnyAvPoli(int x, int y) {
            if (jeVPoli(x, y)) {
                if (mapa[x, y].Data == HodnotaBodu.plne) {
                    return false;
                }
                if (mapa[x, y].Uzavren) {
                    return false;
                }
                return true;
            }
            return false;
        }
        private bool jeVPoli(int x, int y) {
            if (x >= 0 && this.sirkaPole > x) {
                if (y >= 0 && this.vyskaPole > y) {
                    return true;
                }
                return false;
            }
            return false;
        }
        public IEnumerator GetEnumerator() {
            while (!nalezenaCesta) {
                najdiCestuDalsiKrok();
                yield return false;
            }
            yield return true;
        }
        public void Next() {
            najdiCestuDalsiKrok();
        }
        private void najdiCestuDalsiKrok() {
            if (otevrenyBod != null) {
                if (otevrenyBod.Data != HodnotaBodu.start && otevrenyBod.Data != HodnotaBodu.cil) {
                    otevrenyBod.Data = HodnotaBodu.prazdne;
                }
            }
            if (!jizSeHledalo) { // začátek, vložení první položky - startu
                start.OhodnoceniGVzdalenostStart = 0;
                start.OhodnoceniHNejCestaDoCile = Heuristika;
                start.OhodnoceniFNejCestaStartCil = start.OhodnoceniGVzdalenostStart + start.OhodnoceniHNejCestaDoCile;
                otevrene.Add(start);
                vypisDebug(start, "Start");
                jizSeHledalo = true;
            }
            if (otevrene.Count != 0) {
                otevrene.Sort((a, b) => a.OhodnoceniFNejCestaStartCil.CompareTo(b.OhodnoceniFNejCestaStartCil));
                foreach (Bod item in otevrene) {
                    switch (item.Data) {
                        case HodnotaBodu.aktualni:
                            item.Data = HodnotaBodu.prazdne;
                            break;
                        case HodnotaBodu.zkoumane:
                            item.Data = HodnotaBodu.projite;
                            break;
                        default:
                            break;
                    }
                }
                otevrenyBod = otevrene[0];
                if (otevrenyBod.Data != HodnotaBodu.start && otevrenyBod.Data != HodnotaBodu.cil) {
                    otevrenyBod.Data = HodnotaBodu.aktualni;
                }
                if (otevrenyBod == cil) {
                    nalezenaCesta = true;
                    return;
                }
                foreach (Point tah in tahy) {
                    int x = otevrenyBod.Souradnice.X + tah.X;
                    int y = otevrenyBod.Souradnice.Y + tah.Y;
                    if (jePristupnyAvPoli(x, y)) {
                        double noveG = otevrenyBod.OhodnoceniGVzdalenostStart + 1 + Heuristika; // ohodnocení každého bude mám jako by byla vzdálenost 1
                        if (noveG == otevrenyBod.OhodnoceniGVzdalenostStart) {
                            continue;
                        }
                        if (otevrene.Contains(mapa[x, y])) {
                            if (mapa[x, y].OhodnoceniGVzdalenostStart <= noveG) {
                                continue;
                            }
                            break;
                        }
                        if (mapa[x, y].Uzavren == true && mapa[x, y].OhodnoceniGVzdalenostStart <= noveG) {
                            continue;
                        }
                        if (mapa[x, y].Data == HodnotaBodu.prazdne) {
                            mapa[x, y].Data = HodnotaBodu.zkoumane;
                        }
                        mapa[x, y].Rodic = otevrenyBod;
                        mapa[x, y].OhodnoceniGVzdalenostStart = noveG;
                        // metoda prohledávání
                        mapa[x, y].OhodnoceniHNejCestaDoCile = Heuristika * (Math.Abs(x - cil.Souradnice.X) + Math.Abs(y - cil.Souradnice.Y));
                        //mapa[x, y].OhodnoceniHNejCestaDoCile = Heuristika * Math.Sqrt(Math.Pow(x - cil.Souradnice.X, 2) + Math.Pow(y - cil.Souradnice.Y, 2));
                        mapa[x, y].OhodnoceniFNejCestaStartCil = mapa[x, y].OhodnoceniGVzdalenostStart + mapa[x, y].OhodnoceniHNejCestaDoCile;
                        otevrene.Add(mapa[x, y]);
                    }
                }
                otevrene.Remove(otevrenyBod);
                otevrenyBod.Uzavren = true;
                return;
            }
        }
        public LinkedList<Bod> NalezenaCesta() {
            if (nalezenaCesta && cil.Rodic != null) {
                Bod bod = cil;
                cesta = new LinkedList<Bod>();
                while (bod != start) {
                    cesta.AddFirst(bod.Rodic);
                    bod = bod.Rodic;
                }
                int i = 0;
                foreach (Bod item in cesta) {
                    item.PoziceNaCeste = i++;
                }
                return cesta;
            }
            return null;
        }
    }
}
