﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace INZUI_01PSP {
    public partial class KresliciPlocha : UserControl {
        public event EventHandler MouseMoveSpecial;
        Point rozmerKreslicihoPlatna = new Point(360, 360);
        Point rozmeryPolicekPlatna;
        Point rozmeryBodu;

        Color[,] zvyraznena = new Color[15, 15];

        Point pozice = new Point(-1, -1);
        public Point Pozice {
            get { return pozice; }
        }

        bool mouseDown;

        Rectangle ctverec;
        Rectangle ctverecKresleni = new Rectangle();
        Rectangle ctverecOkraj = new Rectangle();

        public KresliciPlocha()
            : this(15, 10) {
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sirkaPole">Počet políček plátna vodorovně</param>
        /// <param name="vyskaPole">Počet políček plátna svisle</param>
        public KresliciPlocha(int x, int y) {
            this.rozmeryPolicekPlatna = new Point(x, y);
            NastavRozmery();
        }

        public void NastavRozmery() {
            this.rozmeryBodu = new Point(rozmerKreslicihoPlatna.X / rozmeryPolicekPlatna.X, rozmerKreslicihoPlatna.Y / rozmeryPolicekPlatna.Y);

            InitializeComponent();
            ctverec = this.Bounds;

            ctverecKresleni.Width = rozmeryBodu.X;
            ctverecKresleni.Height = rozmeryBodu.Y;
            ctverecOkraj.Width = rozmeryBodu.X - 1;
            ctverecOkraj.Height = rozmeryBodu.Y - 1;
        }

        public void SetHighlight(Color highlightColor, int x, int y) {
            zvyraznena[x, y] = highlightColor;
        }

        protected override void OnMouseDown(MouseEventArgs e) {
            base.OnMouseDown(e);
            mouseDown = true;
            this.Invalidate();
        }

        protected override void OnMouseUp(MouseEventArgs e) {
            base.OnMouseUp(e);
            mouseDown = false;
            this.Invalidate();
        }

        protected override void OnMouseLeave(EventArgs e) {
            base.OnMouseLeave(e);
            pozice.X = pozice.Y = -1;
            this.Invalidate();

            if (MouseMoveSpecial != null) {
                MouseMoveSpecial(null, e);
            }
        }

        protected override void OnMouseMove(MouseEventArgs e) {
            base.OnMouseMove(e);
            if (ctverec.Contains(e.Location) == false) {
                if (pozice.X == -1 || pozice.Y == -1) {
                    return;
                }
                pozice.X = pozice.Y = -1;
            } else {
                int posX = e.X / rozmeryBodu.X;
                int posY = e.Y / rozmeryBodu.Y;

                if (posX == pozice.X && posY == pozice.Y) {
                    return;
                }
                pozice.X = posX;
                pozice.Y = posY;
            }
            this.Invalidate();

            if (MouseMoveSpecial != null) {
                MouseMoveSpecial(null, e);
            }
        }

        protected override void OnPaint(PaintEventArgs e) {
            NastavRozmery();
            base.OnPaint(e);
            Graphics graphicsContext = e.Graphics;

            for (int i = 0, startX = 0; i < rozmeryPolicekPlatna.X; i++, startX += rozmeryBodu.X) {
                for (int j = 0, startY = 0; j < rozmeryPolicekPlatna.Y; j++, startY += rozmeryBodu.Y) {
                    ctverecKresleni.X = startX;
                    ctverecKresleni.Y = startY;

                    Color colorHere = zvyraznena[i, j];

                    if (pozice.X == i && pozice.Y == j) {
                        if (mouseDown) {
                            graphicsContext.FillRectangle(Brushes.Black, ctverecKresleni);
                        } else {
                            graphicsContext.FillRectangle(new SolidBrush(colorHere), ctverecKresleni);

                            ctverecOkraj.X = startX;
                            ctverecOkraj.Y = startY;

                            graphicsContext.DrawRectangle(Pens.Black, ctverecOkraj);
                        }
                    } else {
                        if (colorHere != Color.White) {
                            Brush brush = new SolidBrush(colorHere);
                            graphicsContext.FillRectangle(brush, ctverecKresleni);
                        }
                    }
                }
            }
        }
    }
}
